@@ -4,6 +4,7 @@
 #include <assert.h>
 #include <stdexcept>
 #include <qpdf/rijndael.h>
+#include <qpdf/QIntC.hh>
 #include <string>
 #include <stdlib.h>
 
@@ -11,7 +12,7 @@ bool Pl_AES_PDF::use_static_iv = false;
 
 Pl_AES_PDF::Pl_AES_PDF(char const* identifier, Pipeline* next,
 		       bool encrypt, unsigned char const* key,
-                       unsigned int key_bytes) :
+                       size_t key_bytes) :
     Pipeline(identifier, next),
     encrypt(encrypt),
     cbc_mode(true),
@@ -22,31 +23,33 @@ Pl_AES_PDF::Pl_AES_PDF(char const* identifier, Pipeline* next,
     use_specified_iv(false),
     disable_padding(false)
 {
-    unsigned int keybits = 8 * key_bytes;
+    size_t keybits = 8 * key_bytes;
     assert(key_bytes == KEYLENGTH(keybits));
-    this->key = new unsigned char[key_bytes];
-    this->rk = new uint32_t[RKLENGTH(keybits)];
-    unsigned int rk_bytes = RKLENGTH(keybits) * sizeof(uint32_t);
-    std::memcpy(this->key, key, key_bytes);
-    std::memset(this->rk, 0, rk_bytes);
+    this->key = PointerHolder<unsigned char>(
+        true, new unsigned char[key_bytes]);
+    this->rk = PointerHolder<uint32_t>(
+        true, new uint32_t[RKLENGTH(keybits)]);
+    size_t rk_bytes = RKLENGTH(keybits) * sizeof(uint32_t);
+    std::memcpy(this->key.getPointer(), key, key_bytes);
+    std::memset(this->rk.getPointer(), 0, rk_bytes);
     std::memset(this->inbuf, 0, this->buf_size);
     std::memset(this->outbuf, 0, this->buf_size);
     std::memset(this->cbc_block, 0, this->buf_size);
     if (encrypt)
     {
-	this->nrounds = rijndaelSetupEncrypt(this->rk, this->key, keybits);
+	this->nrounds = rijndaelSetupEncrypt(
+            this->rk.getPointer(), this->key.getPointer(), keybits);
     }
     else
     {
-	this->nrounds = rijndaelSetupDecrypt(this->rk, this->key, keybits);
+	this->nrounds = rijndaelSetupDecrypt(
+            this->rk.getPointer(), this->key.getPointer(), keybits);
     }
     assert(this->nrounds == NROUNDS(keybits));
 }
 
 Pl_AES_PDF::~Pl_AES_PDF()
 {
-    delete [] this->key;
-    delete [] this->rk;
 }
 
 void
@@ -68,7 +71,7 @@ Pl_AES_PDF::setIV(unsigned char const* iv, size_t bytes)
     {
         throw std::logic_error(
             "Pl_AES_PDF: specified initialization vector"
-            " size in bytes must be " + QUtil::int_to_string(bytes));
+            " size in bytes must be " + QUtil::uint_to_string(bytes));
     }
     this->use_specified_iv = true;
     memcpy(this->specified_iv, iv, bytes);
@@ -122,7 +125,8 @@ Pl_AES_PDF::finish()
             // Pad as described in section 3.5.1 of version 1.7 of the PDF
             // specification, including providing an entire block of padding
             // if the input was a multiple of 16 bytes.
-            unsigned char pad = (unsigned char) (this->buf_size - this->offset);
+            unsigned char pad =
+                QIntC::to_uchar(this->buf_size - this->offset);
             memset(this->inbuf + this->offset, pad, pad);
             this->offset = this->buf_size;
             flush(false);
@@ -165,7 +169,7 @@ Pl_AES_PDF::initializeVector()
     {
 	for (unsigned int i = 0; i < this->buf_size; ++i)
 	{
-	    this->cbc_block[i] = 14 * (1 + i);
+	    this->cbc_block[i] = static_cast<unsigned char>(14U * (1U + i));
 	}
     }
     else
@@ -220,7 +224,8 @@ Pl_AES_PDF::flush(bool strip_padding)
 		this->inbuf[i] ^= this->cbc_block[i];
 	    }
 	}
-	rijndaelEncrypt(this->rk, this->nrounds, this->inbuf, this->outbuf);
+	rijndaelEncrypt(this->rk.getPointer(),
+                        this->nrounds, this->inbuf, this->outbuf);
 	if (this->cbc_mode)
 	{
 	    memcpy(this->cbc_block, this->outbuf, this->buf_size);
@@ -228,7 +233,8 @@ Pl_AES_PDF::flush(bool strip_padding)
     }
     else
     {
-	rijndaelDecrypt(this->rk, this->nrounds, this->inbuf, this->outbuf);
+	rijndaelDecrypt(this->rk.getPointer(),
+                        this->nrounds, this->inbuf, this->outbuf);
 	if (this->cbc_mode)
 	{
 	    for (unsigned int i = 0; i < this->buf_size; ++i)
