@@ -130,9 +130,9 @@ QPDF::EncryptionData::setV5EncryptionParameters(
 static void
 pad_or_truncate_password_V4(std::string const& password, char k1[key_bytes])
 {
-    int password_bytes = std::min(static_cast<size_t>(key_bytes),
-                                  password.length());
-    int pad_bytes = key_bytes - password_bytes;
+    size_t password_bytes = std::min(QIntC::to_size(key_bytes),
+                                     password.length());
+    size_t pad_bytes = key_bytes - password_bytes;
     memcpy(k1, password.c_str(), password_bytes);
     memcpy(k1 + password_bytes, padding_string, pad_bytes);
 }
@@ -154,9 +154,10 @@ QPDF::trim_user_password(std::string& user_password)
     char const* p2 = 0;
     while ((p2 = strchr(p1, '\x28')) != 0)
     {
-	if (memcmp(p2, padding_string, len - (p2 - cstr)) == 0)
+        size_t idx = toS(p2 - cstr);
+	if (memcmp(p2, padding_string, len - idx) == 0)
 	{
-	    user_password = user_password.substr(0, p2 - cstr);
+	    user_password = user_password.substr(0, idx);
 	    return;
 	}
         else
@@ -183,7 +184,8 @@ truncate_password_V5(std::string const& password)
 }
 
 static void
-iterate_md5_digest(MD5& md5, MD5::Digest& digest, int iterations, int key_len)
+iterate_md5_digest(MD5& md5, MD5::Digest& digest,
+                   int iterations, int key_len)
 {
     md5.digest(digest);
 
@@ -191,29 +193,30 @@ iterate_md5_digest(MD5& md5, MD5::Digest& digest, int iterations, int key_len)
     {
 	MD5 m;
 	m.encodeDataIncrementally(reinterpret_cast<char*>(digest),
-                                  key_len);
+                                  QIntC::to_size(key_len));
 	m.digest(digest);
     }
 }
 
 
 static void
-iterate_rc4(unsigned char* data, int data_len,
+iterate_rc4(unsigned char* data, size_t data_len,
 	    unsigned char* okey, int key_len,
 	    int iterations, bool reverse)
 {
-    unsigned char* key = new unsigned char[key_len];
+    PointerHolder<unsigned char> key_ph = PointerHolder<unsigned char>(
+        true, new unsigned char[QIntC::to_size(key_len)]);
+    unsigned char* key = key_ph.getPointer();
     for (int i = 0; i < iterations; ++i)
     {
 	int const xor_value = (reverse ? iterations - 1 - i : i);
 	for (int j = 0; j < key_len; ++j)
 	{
-	    key[j] = okey[j] ^ xor_value;
+	    key[j] = static_cast<unsigned char>(okey[j] ^ xor_value);
 	}
-	RC4 rc4(key, key_len);
+	RC4 rc4(key, QIntC::to_int(key_len));
 	rc4.process(data, data_len);
     }
-    delete [] key;
 }
 
 static std::string
@@ -228,7 +231,7 @@ process_with_aes(std::string const& key,
     Pl_Buffer buffer("buffer");
     Pl_AES_PDF aes("aes", &buffer, encrypt,
                    QUtil::unsigned_char_pointer(key),
-                   key.length());
+                   QIntC::to_uint(key.length()));
     if (iv)
     {
         aes.setIV(iv, iv_length);
@@ -328,7 +331,7 @@ hash_V5(std::string const& password,
             {
                 unsigned int ch = static_cast<unsigned char>(*(E.rbegin()));
 
-                if (ch <= static_cast<unsigned int>(round_number - 32))
+                if (ch <= QIntC::to_uint(round_number - 32))
                 {
                     done = true;
                 }
@@ -341,7 +344,7 @@ hash_V5(std::string const& password,
 }
 
 static
-void pad_short_parameter(std::string& param, unsigned int max_len)
+void pad_short_parameter(std::string& param, size_t max_len)
 {
     if (param.length() < max_len)
     {
@@ -367,11 +370,11 @@ QPDF::compute_data_key(std::string const& encryption_key,
     }
 
     // Append low three bytes of object ID and low two bytes of generation
-    result += static_cast<char>(objid & 0xff);
-    result += static_cast<char>((objid >> 8) & 0xff);
-    result += static_cast<char>((objid >> 16) & 0xff);
-    result += static_cast<char>(generation & 0xff);
-    result += static_cast<char>((generation >> 8) & 0xff);
+    result.append(1, static_cast<char>(objid & 0xff));
+    result.append(1, static_cast<char>((objid >> 8) & 0xff));
+    result.append(1, static_cast<char>((objid >> 16) & 0xff));
+    result.append(1, static_cast<char>(generation & 0xff));
+    result.append(1, static_cast<char>((generation >> 8) & 0xff));
     if (use_aes)
     {
 	result += "sAlT";
@@ -382,7 +385,7 @@ QPDF::compute_data_key(std::string const& encryption_key,
     MD5::Digest digest;
     md5.digest(digest);
     return std::string(reinterpret_cast<char*>(digest),
-		       std::min(result.length(), static_cast<size_t>(16)));
+		       std::min(result.length(), toS(16)));
 }
 
 std::string
@@ -437,10 +440,11 @@ QPDF::compute_encryption_key_from_password(
 	md5.encodeDataIncrementally(bytes, 4);
     }
     MD5::Digest digest;
-    int key_len = std::min(static_cast<int>(sizeof(digest)),
-                           data.getLengthBytes());
+    int key_len =
+        std::min(QIntC::to_int(sizeof(digest)), data.getLengthBytes());
     iterate_md5_digest(md5, digest, ((data.getR() >= 3) ? 50 : 0), key_len);
-    return std::string(reinterpret_cast<char*>(digest), key_len);
+    return std::string(reinterpret_cast<char*>(digest),
+                       QIntC::to_size(key_len));
 }
 
 static void
@@ -463,7 +467,7 @@ compute_O_rc4_key(std::string const& user_password,
     md5.encodeDataIncrementally(
 	pad_or_truncate_password_V4(password).c_str(), key_bytes);
     MD5::Digest digest;
-    int key_len = std::min(static_cast<int>(sizeof(digest)),
+    int key_len = std::min(QIntC::to_int(sizeof(digest)),
                            data.getLengthBytes());
     iterate_md5_digest(md5, digest, ((data.getR() >= 3) ? 50 : 0), key_len);
     memcpy(key, digest, OU_key_bytes_V4);
@@ -482,7 +486,7 @@ compute_O_value(std::string const& user_password,
     char upass[key_bytes];
     pad_or_truncate_password_V4(user_password, upass);
     std::string k1(reinterpret_cast<char*>(O_key), OU_key_bytes_V4);
-    pad_short_parameter(k1, data.getLengthBytes());
+    pad_short_parameter(k1, QIntC::to_size(data.getLengthBytes()));
     iterate_rc4(QUtil::unsigned_char_pointer(upass), key_bytes,
 		O_key, data.getLengthBytes(),
                 (data.getR() >= 3) ? 20 : 1, false);
@@ -499,7 +503,7 @@ compute_U_value_R2(std::string const& user_password,
     std::string k1 = QPDF::compute_encryption_key(user_password, data);
     char udata[key_bytes];
     pad_or_truncate_password_V4("", udata);
-    pad_short_parameter(k1, data.getLengthBytes());
+    pad_short_parameter(k1, QIntC::to_size(data.getLengthBytes()));
     iterate_rc4(QUtil::unsigned_char_pointer(udata), key_bytes,
 		QUtil::unsigned_char_pointer(k1),
                 data.getLengthBytes(), 1, false);
@@ -521,7 +525,7 @@ compute_U_value_R3(std::string const& user_password,
                                 data.getId1().length());
     MD5::Digest digest;
     md5.digest(digest);
-    pad_short_parameter(k1, data.getLengthBytes());
+    pad_short_parameter(k1, QIntC::to_size(data.getLengthBytes()));
     iterate_rc4(digest, sizeof(MD5::Digest),
 		QUtil::unsigned_char_pointer(k1),
                 data.getLengthBytes(), 20, false);
@@ -555,8 +559,8 @@ check_user_password_V4(std::string const& user_password,
     // Algorithm 3.6 from the PDF 1.7 Reference Manual
 
     std::string u_value = compute_U_value(user_password, data);
-    int to_compare = ((data.getR() >= 3) ? sizeof(MD5::Digest)
-                      : key_bytes);
+    size_t to_compare = ((data.getR() >= 3) ? sizeof(MD5::Digest)
+                         : key_bytes);
     return (memcmp(data.getU().c_str(), u_value.c_str(), to_compare) == 0);
 }
 
@@ -598,7 +602,7 @@ check_owner_password_V4(std::string& user_password,
     unsigned char O_data[key_bytes];
     memcpy(O_data, QUtil::unsigned_char_pointer(data.getO()), key_bytes);
     std::string k1(reinterpret_cast<char*>(key), OU_key_bytes_V4);
-    pad_short_parameter(k1, data.getLengthBytes());
+    pad_short_parameter(k1, QIntC::to_size(data.getLengthBytes()));
     iterate_rc4(O_data, key_bytes, QUtil::unsigned_char_pointer(k1),
                 data.getLengthBytes(),
                 (data.getR() >= 3) ? 20 : 1, true);
@@ -694,7 +698,8 @@ compute_Perms_value_V5_clear(std::string const& encryption_key,
                              unsigned char k[16])
 {
     // From algorithm 3.10 from the PDF 1.7 extension level 3
-    unsigned long long extended_perms = 0xffffffff00000000LL | data.getP();
+    unsigned long long extended_perms =
+        0xffffffff00000000LL | static_cast<unsigned long long>(data.getP());
     for (int i = 0; i < 8; ++i)
     {
         k[i] = static_cast<unsigned char>(extended_perms & 0xff);
@@ -764,14 +769,15 @@ QPDF::recover_encryption_key_with_password(
 }
 
 QPDF::encryption_method_e
-QPDF::interpretCF(QPDFObjectHandle cf)
+QPDF::interpretCF(
+    PointerHolder<EncryptionParameters> encp, QPDFObjectHandle cf)
 {
     if (cf.isName())
     {
 	std::string filter = cf.getName();
-	if (this->m->crypt_filters.count(filter) != 0)
+	if (encp->crypt_filters.count(filter) != 0)
 	{
-	    return this->m->crypt_filters[filter];
+	    return encp->crypt_filters[filter];
 	}
 	else if (filter == "/Identity")
 	{
@@ -792,11 +798,11 @@ QPDF::interpretCF(QPDFObjectHandle cf)
 void
 QPDF::initializeEncryption()
 {
-    if (this->m->encryption_initialized)
+    if (this->m->encp->encryption_initialized)
     {
 	return;
     }
-    this->m->encryption_initialized = true;
+    this->m->encp->encryption_initialized = true;
 
     // After we initialize encryption parameters, we must used stored
     // key information and never look at /Encrypt again.  Otherwise,
@@ -811,7 +817,7 @@ QPDF::initializeEncryption()
     // Go ahead and set this->m->encrypted here.  That way, isEncrypted
     // will return true even if there were errors reading the
     // encryption dictionary.
-    this->m->encrypted = true;
+    this->m->encp->encrypted = true;
 
     std::string id1;
     QPDFObjectHandle id_obj = this->m->trailer.getKey("/ID");
@@ -867,11 +873,11 @@ QPDF::initializeEncryption()
 		      "or the wrong type");
     }
 
-    int V = encryption_dict.getKey("/V").getIntValue();
-    int R = encryption_dict.getKey("/R").getIntValue();
+    int V = encryption_dict.getKey("/V").getIntValueAsInt();
+    int R = encryption_dict.getKey("/R").getIntValueAsInt();
     std::string O = encryption_dict.getKey("/O").getStringValue();
     std::string U = encryption_dict.getKey("/U").getStringValue();
-    unsigned int P = encryption_dict.getKey("/P").getIntValue();
+    int P = encryption_dict.getKey("/P").getIntValueAsInt();
 
     // If supporting new encryption R/V values, remember to update
     // error message inside this if statement.
@@ -885,8 +891,8 @@ QPDF::initializeEncryption()
                       QUtil::int_to_string(V) + " (max 5)");
     }
 
-    this->m->encryption_V = V;
-    this->m->encryption_R = R;
+    this->m->encp->encryption_V = V;
+    this->m->encp->encryption_R = R;
 
     // OE, UE, and Perms are only present if V >= 5.
     std::string OE;
@@ -931,28 +937,44 @@ QPDF::initializeEncryption()
         pad_short_parameter(Perms, Perms_key_bytes_V5);
     }
 
-    int Length = 40;
-    if (encryption_dict.getKey("/Length").isInteger())
+    int Length = 0;
+    if (V <= 1)
     {
-	Length = encryption_dict.getKey("/Length").getIntValue();
-        if (R < 3)
+        Length = 40;
+    }
+    else if (V == 4)
+    {
+        Length = 128;
+    }
+    else if (V == 5)
+    {
+        Length = 256;
+    }
+    else
+    {
+        if (encryption_dict.getKey("/Length").isInteger())
         {
-            // Force Length to 40 regardless of what the file says.
-            Length = 40;
+            Length = encryption_dict.getKey("/Length").getIntValueAsInt();
+            if ((Length % 8) || (Length < 40) || (Length > 128))
+            {
+                Length = 0;
+            }
         }
-	if ((Length % 8) || (Length < 40) || (Length > 256))
-	{
-	    throw QPDFExc(qpdf_e_damaged_pdf, this->m->file->getName(),
-			  "encryption dictionary",
-                          this->m->file->getLastOffset(),
-			  "invalid /Length value in encryption dictionary");
-	}
+        if (Length == 0)
+        {
+            Length = 128;
+        }
+    }
+    if (Length == 0)
+    {
+        // Still no Length? Just take a guess.
+        Length = 128;
     }
 
-    this->m->encrypt_metadata = true;
+    this->m->encp->encrypt_metadata = true;
     if ((V >= 4) && (encryption_dict.getKey("/EncryptMetadata").isBool()))
     {
-	this->m->encrypt_metadata =
+	this->m->encp->encrypt_metadata =
 	    encryption_dict.getKey("/EncryptMetadata").getBoolValue();
     }
 
@@ -993,58 +1015,83 @@ QPDF::initializeEncryption()
 			method = e_unknown;
 		    }
 		}
-		this->m->crypt_filters[filter] = method;
+		this->m->encp->crypt_filters[filter] = method;
 	    }
 	}
 
 	QPDFObjectHandle StmF = encryption_dict.getKey("/StmF");
 	QPDFObjectHandle StrF = encryption_dict.getKey("/StrF");
 	QPDFObjectHandle EFF = encryption_dict.getKey("/EFF");
-	this->m->cf_stream = interpretCF(StmF);
-	this->m->cf_string = interpretCF(StrF);
+	this->m->encp->cf_stream = interpretCF(this->m->encp, StmF);
+	this->m->encp->cf_string = interpretCF(this->m->encp, StrF);
 	if (EFF.isName())
 	{
-	    this->m->cf_file = interpretCF(EFF);
+	    this->m->encp->cf_file = interpretCF(this->m->encp, EFF);
 	}
 	else
 	{
-	    this->m->cf_file = this->m->cf_stream;
+	    this->m->encp->cf_file = this->m->encp->cf_stream;
 	}
     }
 
-    EncryptionData data(V, R, Length / 8, P, O, U, OE, UE, Perms,
-                        id1, this->m->encrypt_metadata);
+    EncryptionData data(V, R, Length / 8,
+                        P, O, U, OE, UE, Perms,
+                        id1, this->m->encp->encrypt_metadata);
     if (this->m->provided_password_is_hex_key)
     {
         // ignore passwords in file
     }
-    else if (check_owner_password(
-                 this->m->user_password, this->m->provided_password, data))
-    {
-	// password supplied was owner password; user_password has
-	// been initialized for V < 5
-    }
-    else if (check_user_password(this->m->provided_password, data))
-    {
-	this->m->user_password = this->m->provided_password;
-    }
     else
     {
-	throw QPDFExc(qpdf_e_password, this->m->file->getName(),
-		      "", 0, "invalid password");
+        this->m->encp->owner_password_matched = check_owner_password(
+            this->m->encp->user_password,
+            this->m->encp->provided_password, data);
+        if (this->m->encp->owner_password_matched && (V < 5))
+        {
+            // password supplied was owner password; user_password has
+            // been initialized for V < 5
+            if (getTrimmedUserPassword() == this->m->encp->provided_password)
+            {
+                this->m->encp->user_password_matched = true;
+                QTC::TC("qpdf", "QPDF_encryption user matches owner V < 5");
+            }
+        }
+        else
+        {
+            this->m->encp->user_password_matched = check_user_password(
+                this->m->encp->provided_password, data);
+            if (this->m->encp->user_password_matched)
+            {
+                this->m->encp->user_password =
+                    this->m->encp->provided_password;
+            }
+        }
+        if (this->m->encp->user_password_matched &&
+            this->m->encp->owner_password_matched)
+        {
+            QTC::TC("qpdf", "QPDF_encryption same password",
+                    (V < 5) ? 0 : 1);
+        }
+        if (! (this->m->encp->owner_password_matched ||
+               this->m->encp->user_password_matched))
+        {
+            throw QPDFExc(qpdf_e_password, this->m->file->getName(),
+                          "", 0, "invalid password");
+        }
     }
 
     if (this->m->provided_password_is_hex_key)
     {
-        this->m->encryption_key = QUtil::hex_decode(this->m->provided_password);
+        this->m->encp->encryption_key =
+            QUtil::hex_decode(this->m->encp->provided_password);
     }
     else if (V < 5)
     {
         // For V < 5, the user password is encrypted with the owner
         // password, and the user password is always used for
         // computing the encryption key.
-        this->m->encryption_key = compute_encryption_key(
-            this->m->user_password, data);
+        this->m->encp->encryption_key = compute_encryption_key(
+            this->m->encp->user_password, data);
     }
     else
     {
@@ -1052,8 +1099,8 @@ QPDF::initializeEncryption()
         // compute the encryption key, and neither password can be
         // used to recover the other.
         bool perms_valid;
-        this->m->encryption_key = recover_encryption_key_with_password(
-            this->m->provided_password, data, perms_valid);
+        this->m->encp->encryption_key = recover_encryption_key_with_password(
+            this->m->encp->provided_password, data, perms_valid);
         if (! perms_valid)
         {
             warn(QPDFExc(qpdf_e_damaged_pdf, this->m->file->getName(),
@@ -1066,26 +1113,28 @@ QPDF::initializeEncryption()
 }
 
 std::string
-QPDF::getKeyForObject(int objid, int generation, bool use_aes)
+QPDF::getKeyForObject(
+    PointerHolder<EncryptionParameters> encp,
+    int objid, int generation, bool use_aes)
 {
-    if (! this->m->encrypted)
+    if (! encp->encrypted)
     {
 	throw std::logic_error(
 	    "request for encryption key in non-encrypted PDF");
     }
 
-    if (! ((objid == this->m->cached_key_objid) &&
-	   (generation == this->m->cached_key_generation)))
+    if (! ((objid == encp->cached_key_objid) &&
+	   (generation == encp->cached_key_generation)))
     {
-	this->m->cached_object_encryption_key =
-	    compute_data_key(this->m->encryption_key, objid, generation,
-                             use_aes, this->m->encryption_V,
-                             this->m->encryption_R);
-	this->m->cached_key_objid = objid;
-	this->m->cached_key_generation = generation;
+	encp->cached_object_encryption_key =
+	    compute_data_key(encp->encryption_key, objid, generation,
+                             use_aes, encp->encryption_V,
+                             encp->encryption_R);
+	encp->cached_key_objid = objid;
+	encp->cached_key_generation = generation;
     }
 
-    return this->m->cached_object_encryption_key;
+    return encp->cached_object_encryption_key;
 }
 
 void
@@ -1096,9 +1145,9 @@ QPDF::decryptString(std::string& str, int objid, int generation)
 	return;
     }
     bool use_aes = false;
-    if (this->m->encryption_V >= 4)
+    if (this->m->encp->encryption_V >= 4)
     {
-	switch (this->m->cf_string)
+	switch (this->m->encp->cf_string)
 	{
 	  case e_none:
 	    return;
@@ -1123,13 +1172,14 @@ QPDF::decryptString(std::string& str, int objid, int generation)
 			 " strings may be decrypted improperly"));
 	    // To avoid repeated warnings, reset cf_string.  Assume
 	    // we'd want to use AES if V == 4.
-	    this->m->cf_string = e_aes;
+	    this->m->encp->cf_string = e_aes;
             use_aes = true;
 	    break;
 	}
     }
 
-    std::string key = getKeyForObject(objid, generation, use_aes);
+    std::string key = getKeyForObject(
+        this->m->encp, objid, generation, use_aes);
     try
     {
 	if (use_aes)
@@ -1148,11 +1198,11 @@ QPDF::decryptString(std::string& str, int objid, int generation)
 	else
 	{
 	    QTC::TC("qpdf", "QPDF_encryption rc4 decode string");
-	    unsigned int vlen = str.length();
+	    size_t vlen = str.length();
 	    // Using PointerHolder guarantees that tmp will
 	    // be freed even if rc4.process throws an exception.
 	    PointerHolder<char> tmp(true, QUtil::copy_string(str));
-	    RC4 rc4(QUtil::unsigned_char_pointer(key), key.length());
+	    RC4 rc4(QUtil::unsigned_char_pointer(key), toI(key.length()));
 	    rc4.process(QUtil::unsigned_char_pointer(tmp.getPointer()), vlen);
 	    str = std::string(tmp.getPointer(), vlen);
 	}
@@ -1173,8 +1223,12 @@ QPDF::decryptString(std::string& str, int objid, int generation)
 }
 
 void
-QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
+QPDF::decryptStream(PointerHolder<EncryptionParameters> encp,
+                    PointerHolder<InputSource> file,
+                    QPDF& qpdf_for_warning, Pipeline*& pipeline,
+                    int objid, int generation,
 		    QPDFObjectHandle& stream_dict,
+                    bool is_attachment_stream,
 		    std::vector<PointerHolder<Pipeline> >& heap)
 {
     std::string type;
@@ -1188,7 +1242,7 @@ QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
 	return;
     }
     bool use_aes = false;
-    if (this->m->encryption_V >= 4)
+    if (encp->encryption_V >= 4)
     {
 	encryption_method_e method = e_unknown;
 	std::string method_source = "/StmF from /Encrypt dictionary";
@@ -1204,7 +1258,7 @@ QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
                      "/CryptFilterDecodeParms"))
                 {
                     QTC::TC("qpdf", "QPDF_encryption stream crypt filter");
-                    method = interpretCF(decode_parms.getKey("/Name"));
+                    method = interpretCF(encp, decode_parms.getKey("/Name"));
                     method_source = "stream's Crypt decode parameters";
                 }
             }
@@ -1227,7 +1281,7 @@ QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
                             {
                                 QTC::TC("qpdf", "QPDF_encrypt crypt array");
                                 method = interpretCF(
-                                    crypt_params.getKey("/Name"));
+                                    encp, crypt_params.getKey("/Name"));
                                 method_source = "stream's Crypt "
                                     "decode parameters (array)";
                             }
@@ -1239,21 +1293,21 @@ QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
 
 	if (method == e_unknown)
 	{
-	    if ((! this->m->encrypt_metadata) && (type == "/Metadata"))
+	    if ((! encp->encrypt_metadata) && (type == "/Metadata"))
 	    {
 		QTC::TC("qpdf", "QPDF_encryption cleartext metadata");
 		method = e_none;
 	    }
 	    else
 	    {
-                if (this->m->attachment_streams.count(
-                        QPDFObjGen(objid, generation)) > 0)
+                if (is_attachment_stream)
                 {
-                    method = this->m->cf_file;
+                    QTC::TC("qpdf", "QPDF_encryption attachment stream");
+                    method = encp->cf_file;
                 }
                 else
                 {
-                    method = this->m->cf_stream;
+                    method = encp->cf_stream;
                 }
 	    }
 	}
@@ -1277,19 +1331,20 @@ QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
 
 	  default:
 	    // filter local to this stream.
-	    warn(QPDFExc(qpdf_e_damaged_pdf, this->m->file->getName(),
-			 this->m->last_object_description,
-			 this->m->file->getLastOffset(),
-			 "unknown encryption filter for streams"
-			 " (check " + method_source + ");"
-			 " streams may be decrypted improperly"));
+	    qpdf_for_warning.warn(
+                QPDFExc(qpdf_e_damaged_pdf, file->getName(),
+                        "", file->getLastOffset(),
+                        "unknown encryption filter for streams"
+                        " (check " + method_source + ");"
+                        " streams may be decrypted improperly"));
 	    // To avoid repeated warnings, reset cf_stream.  Assume
 	    // we'd want to use AES if V == 4.
-	    this->m->cf_stream = e_aes;
+	    encp->cf_stream = e_aes;
+            use_aes = true;
 	    break;
 	}
     }
-    std::string key = getKeyForObject(objid, generation, use_aes);
+    std::string key = getKeyForObject(encp, objid, generation, use_aes);
     if (use_aes)
     {
 	QTC::TC("qpdf", "QPDF_encryption aes decode stream");
@@ -1302,7 +1357,7 @@ QPDF::decryptStream(Pipeline*& pipeline, int objid, int generation,
 	QTC::TC("qpdf", "QPDF_encryption rc4 decode stream");
 	pipeline = new Pl_RC4("RC4 stream decryption", pipeline,
 			      QUtil::unsigned_char_pointer(key),
-                              key.length());
+                              toI(key.length()));
     }
     heap.push_back(pipeline);
 }
@@ -1349,13 +1404,13 @@ QPDF::compute_encryption_parameters_V5(
 std::string const&
 QPDF::getPaddedUserPassword() const
 {
-    return this->m->user_password;
+    return this->m->encp->user_password;
 }
 
 std::string
 QPDF::getTrimmedUserPassword() const
 {
-    std::string result = this->m->user_password;
+    std::string result = this->m->encp->user_password;
     trim_user_password(result);
     return result;
 }
@@ -1363,13 +1418,13 @@ QPDF::getTrimmedUserPassword() const
 std::string
 QPDF::getEncryptionKey() const
 {
-    return this->m->encryption_key;
+    return this->m->encp->encryption_key;
 }
 
 bool
 QPDF::isEncrypted() const
 {
-    return this->m->encrypted;
+    return this->m->encp->encrypted;
 }
 
 bool
@@ -1386,19 +1441,19 @@ QPDF::isEncrypted(int& R, int& P, int& V,
                   encryption_method_e& string_method,
                   encryption_method_e& file_method)
 {
-    if (this->m->encrypted)
+    if (this->m->encp->encrypted)
     {
 	QPDFObjectHandle trailer = getTrailer();
 	QPDFObjectHandle encrypt = trailer.getKey("/Encrypt");
 	QPDFObjectHandle Pkey = encrypt.getKey("/P");
 	QPDFObjectHandle Rkey = encrypt.getKey("/R");
         QPDFObjectHandle Vkey = encrypt.getKey("/V");
-	P = Pkey.getIntValue();
-	R = Rkey.getIntValue();
-        V = Vkey.getIntValue();
-        stream_method = this->m->cf_stream;
-        string_method = this->m->cf_string;
-        file_method = this->m->cf_file;
+	P = Pkey.getIntValueAsInt();
+	R = Rkey.getIntValueAsInt();
+        V = Vkey.getIntValueAsInt();
+        stream_method = this->m->encp->cf_stream;
+        string_method = this->m->encp->cf_string;
+        file_method = this->m->encp->cf_file;
 	return true;
     }
     else
@@ -1407,11 +1462,23 @@ QPDF::isEncrypted(int& R, int& P, int& V,
     }
 }
 
+bool
+QPDF::ownerPasswordMatched() const
+{
+    return this->m->encp->owner_password_matched;
+}
+
+bool
+QPDF::userPasswordMatched() const
+{
+    return this->m->encp->user_password_matched;
+}
+
 static bool
 is_bit_set(int P, int bit)
 {
     // Bits in P are numbered from 1 in the spec
-    return (P & (1 << (bit - 1)));
+    return ((P & (1 << (bit - 1))) != 0);
 }
 
 bool
