@@ -6,6 +6,7 @@
 #include <qpdf/BufferInputSource.hh>
 #include <qpdf/QPDF.hh>
 #include <qpdf/Pl_Buffer.hh>
+#include <qpdf/QIntC.hh>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -46,7 +47,7 @@ Finder::check()
     QPDFTokenizer::Token t = tokenizer.readToken(is, "finder", true);
     qpdf_offset_t offset = this->is->tell();
     bool result = (t == QPDFTokenizer::Token(QPDFTokenizer::tt_word, str));
-    this->is->seek(offset - this->str.length(), SEEK_SET);
+    this->is->seek(offset - QIntC::to_offset(this->str.length()), SEEK_SET);
     return result;
 }
 
@@ -170,7 +171,7 @@ dump_tokens(PointerHolder<InputSource> is, std::string const& label,
                 std::cout << " (raw: " << sanitize(token.getRawValue()) << ")";
             }
         }
-        if (token.getType() == QPDFTokenizer::tt_bad)
+        if (! token.getErrorMessage().empty())
         {
             std::cout << " (" << token.getErrorMessage() << ")";
         }
@@ -183,7 +184,9 @@ dump_tokens(PointerHolder<InputSource> is, std::string const& label,
         else if (skip_inline_images &&
                  (token == QPDFTokenizer::Token(QPDFTokenizer::tt_word, "ID")))
         {
-            tokenizer.expectInlineImage();
+            char ch;
+            is->read(&ch, 1);
+            tokenizer.expectInlineImage(is);
             inline_image_offset = is->tell();
         }
         else if (token.getType() == QPDFTokenizer::tt_eof)
@@ -276,7 +279,7 @@ int main(int argc, char* argv[])
                 {
                     usage();
                 }
-                max_len = QUtil::string_to_int(argv[i]);
+                max_len = QUtil::string_to_uint(argv[i]);
             }
             else if (strcmp(argv[i], "-no-ignorable") == 0)
             {
